setwd("C:/cwru/classes/epbi473_05/lec4(riskModels)")
rm(list=ls(all=TRUE))
cols=c("city","sex","doseg","agexg","calg","kerma","PY","adjPY","num.entering",
"age","agex","tsx","cal","sv","gam","neut","lymphoma","NHL","leukemia","AML","ALL","CML","ATL","MM")      
d<-read.table("c:/abomb/hema87.dat", header=F,col.names=cols);
d=d[d$adjPY>0,] #remove two recs with zero py
d=d[d$kerma==1,] # take only kerma < 4 Gy
d=d[d$city==1,] #Take only Hiroshima fit
d=d[d$sex==1,] #take only males
attach(d)
#summary(d)
py=10^4*adjPY

# ************************** Block 1 (let the IR-to-CML WAITING TIME distribution shape speak) *****************************************************

X0=c(c1=-13,k=0.05,rep(-10,10));X0 #am km L
fL<-function(x){
 c1=x[1];k=x[2];L=x[3:12];   		
 mn = exp(c1+k*age)*py + sv*exp(L[calg])*py;
 -sum(CML*log(mn) - mn)
}
solL=optim(X0,fL,method="L-BFGS-B",control=list(maxit=400));solL  # the idea here is to let the fit speak through L, a bit like a one way anova
#
fixed=c(T,T,F,F)
fixed=c(F,F,F,F)
X0=c(c1=-12.6,k=0.042,c2=-8,kt=0.4)
flingam2<-function(x){
 x[fixed]=X0[fixed]
 c1=x[1];k=x[2];c2=x[3];kt=x[4];   		
 mn = exp(c1+k*age)*py + sv*(tsx^2)*exp(c2-kt*tsx)*py;
 -sum(CML*log(mn) - mn)
}
solg=optim(X0,flingam2,method="L-BFGS-B",control=list(maxit=400));solg
#          c1            k           c2           kt 
#-12.78514425   0.04062961  -8.39283620   0.40370903 
tf=seq(0,45,0.1)
w=(tf^2)*exp(solg$par[3]-solg$par[4]*tf)
wait=exp(solL$par[3:12])
year1=tf+1945
year2=c(1951,1953,1956,1959,1963,1968,1973,1978,1983,1987)
#year2=c(1951,1953,1956,1959,1963)
plot(year1,w,type="l",xlab="Year",ylab="w(t)",main="IR-to-CML HM waiting time pdf shape")
points(year2,wait,pch=5)
# solg$par = -12.51729945   0.01462043  -8.49996022   0.38101333

# *************************** Block 2 (let the DOSE RESPONSE shape speak) ****************************************************

svc=cut(sv,c(-1,.02,.2,.5,1,2,10))
table(svc[CML>0])   # a decent spread of cases
mdose=c(0,.1,.35,.75,1.5,3)
fixed=c(T,T,T,T,rep(F,5))
fixed=c(F,F,F,T,rep(F,5))
D0=c(-40,rep(-5,5));names(D0)<-paste("Ds",1:6,sep="");D0
X0=c(c1=-12.6,k=0.042,kt=0.411,D0)
fD<-function(x){
 x[fixed]=X0[fixed]
 c1=x[1];k=x[2];kt=x[3];Ds=x[4:9];   		
 mn = exp(c1+k*age)*py + exp(Ds[svc])*(tsx^2)*exp(-kt*tsx)*py;
 -sum(CML*log(mn) - mn)
 }
solD=optim(X0,fD,method="L-BFGS-B",control=list(maxit=400))
solD
# background loose, first dose fixed
#         c1           k          kt         Ds1         Ds2         Ds3         Ds4         Ds5         Ds6 
#-12.6289500   0.0386758   0.4061382 -40.0000000 -14.3249094  -9.1625175  -8.9581512  -7.6438053  -7.0405166 
tf=seq(0,45,0.1)
w=(tf^2)*exp(-solD$par[3]*tf)
year1=tf+1945
plot(year1,w,type="l",xlab="Year",ylab="w(t)",main="IR-to-CML HM waiting time pdf shape")

ampsD=exp(solD$par[4:9])

plot(mdose,ampsD,pch=5,xlab="Sv",ylab="amplitude",main="Excess CML risk dose-response shape")
#lines(mdose,exp(solg$par[3])*mdose,col="blue") # not sure why this is slightly off
lmD=lm(ampsD~mdose-1)
abline(lmD)
slope=lmD$coeff["mdose"];slope
cmlRiskD=slope*sum(w)*.1
cmlRiskD  # this is the lifetime excess risk of CML after a 1 Sv exposure. Constrained Background increases the risk since it drops the back incid at young ages. 
text(1,8e-4,paste("Lifetime CML risk w/o competing risks =",format(cmlRiskD,digits=2),"per Sv"))

# *************************** Block 3 (let AGE AT EXPOSURE influence on ER (excess risk) speak) ****************************************************

# now try letting age at exposure speak
agexc=cut(agex,c(-1,10,20,30,40,50,100)) 
agexc=cut(agex,5) 
agexc=cut(agex,c(-1,20,30,40,100)) 
table(agexc[CML>0])   # check spread of cases

magex=c(10,25,35,50)
#magex=c(5,15,25,35,45,55)
n=length(magex)
X0=c(c1=-12.6,k=0.042,kt=0.411,rep(-2,n))  
fixed=c(T,T,T,rep(F,4))
fixed=c(rep(F,7)) # comment this one to fix the first 3 params defined above to ICs 
fD<-function(x){
x[fixed]=X0[fixed]
c1=x[1];k=x[2];kt=x[3];A=x[4:(n+3)]   		
mn = exp(c1+k*age)*py + sv*(tsx^2)*exp(A[agexc]-kt*tsx)*py;
-sum(CML*log(mn) - mn)
}
solX=optim(X0,fD,method="L-BFGS-B",control=list(maxit=400));solX
#        c1          k         kt                                             
#-12.600000   0.042000   0.380000  -9.093278  -7.452939  -8.018901  -9.329662 
amps=exp(solX$par[4:(n+3)])
plot(magex,amps,pch=5,xlab="Age at exposure",ylab="amplitude",main="Excess CML risk versus age-at-exposure response shape")
# 
#***************************** Block 4 (let BOTH DOSE and WAITING TIME (latency L) speak*********************************************

# this is analogous to a two-way ANOVA situation
svc=cut(sv,c(-1,.02,.2,.5,1,10))
mdose=c(0,.1,.35,.75,2.5)
table(svc[CML>0])   # get 7 in highest Dose group to estimate L shape there. NOTE: very few background cases => tough to get background incidence here
table(calg[CML>0])   # a decent spread of cases, most at early times
table(svc[CML>0],calg[CML>0])
#             1 2 3 4 5 6 7 8 10
#  (-1,0.02]  0 0 0 0 1 2 3 1  1
#  (0.02,0.2] 0 1 1 0 0 0 0 1  1
#  (0.2,0.5]  1 0 1 1 1 1 0 1  0
#  (0.5,1]    1 1 0 0 0 0 0 0  0
#  (1,10]     3 2 1 1 0 0 0 0  0
D0=c(-10,-5,-5,-5);names(D0)<-paste("Ds",1:4,sep="");D0
L0=rep(-7,10);names(L0)<-paste("Ls",1:10,sep="");L0
X0=c(c1=-12.6,k=.042,D0,L0);X0  
fixed=c(rep(T,2),F,rep(F,8))             
fixed=c(rep(F,2),F,rep(F,8))             
fLD<-function(x){
x[fixed]=X0[fixed]
# third param fixed => Ds[1]=IC 
c1=x[1];k=x[2];Ds=x[3:6];L=x[7:16]  
Ds[5]=0  # let the highest dose group determine the baseline L values. Note that this is not a parameter, it's an add on to a length 4 vector. 
mn = exp(c1+k*age)*py + exp(Ds[svc]+L[calg])*py;
-sum(CML*log(mn) - mn)
}
solLD=optim(X0,fLD,method="L-BFGS-B",control=list(maxit=400));solLD
#          c1            k          Ds1          Ds2          Ds3          Ds4          Ls1          Ls2          Ls3          Ls4          Ls5          Ls6 
#-13.02535551   0.04402153 -10.10794308  -7.39815478  -1.54075242  -1.57490518  -6.32268424  -6.66569978  -7.55872482  -7.40768581  -8.63255889 -10.24890339 
#         Ls7          Ls8          Ls9         Ls10 
#-11.90072604  -8.30070819 -11.62304016 -11.04050025 
wait=exp(solLD$par[7:16])
year2=c(1951,1953,1956,1959,1963)
year2=c(1951,1953,1956,1959,1963,1968,1973,1978,1983,1987)
plot(year2,wait,type="p",xlab="Year",ylab="w(t)",main="IR-to-CML HM waiting time pdf shape")
locator(n=1)
amps=c(exp(solLD$par[3:6]),1)
plot(mdose,amps,pch=5,xlab="Sv",ylab="amplitude",main="Excess risk dose-response shape")


#***************************** Block 5 (Check how well HM CML model fits HM CML data) *********************************************
fixed=c(F,F,F,F)
X0=c(c1=-12.6,k=0.042,c2=-8,kt=0.4)
flingam2<-function(x){
 x[fixed]=X0[fixed]
 c1=x[1];k=x[2];c2=x[3];kt=x[4];   		
 mn = exp(c1+k*age)*py + sv*(tsx^2)*exp(c2-kt*tsx)*py;
 -sum(CML*log(mn) - mn)
}
solg=optim(X0,flingam2,method="L-BFGS-B",control=list(maxit=400));
solg
#          c1            k           c2           kt 
#-12.78514425   0.04062961  -8.39283620   0.40370903 
attach(as.list(solg$par))

ecml=exp(c1+k*age)*py + sv*(tsx^2)*exp(c2-kt*tsx)*py # this is a big long vector! it gets summed below like observed counts

# need wide intervals since only HM are here
agec=cut(age,seq(0,80,20))
agem=seq(10,70,20)  

svc=cut(sv,c(-1,.01,.4,10),labels=c("low","med","high"))
pyt=tapply(py,list(svc,agec),sum,na.rm=TRUE);pyt
rb=c("red","orange","green")
agef=seq(0.5,87.5,by=1)
canc=d[,"CML"]
cancerCount=tapply(canc,list(svc,agec),sum,na.rm=TRUE)
bk=cancerCount[1,]/pyt[1,] #;bk
med=cancerCount[2,]/pyt[2,]
hi=cancerCount[3,]/pyt[3,]
EcancerCount=tapply(ecml,list(svc,agec),sum,na.rm=TRUE)
Ebk=EcancerCount[1,]/pyt[1,] #;bk
Emed=EcancerCount[2,]/pyt[2,]
Ehi=EcancerCount[3,]/pyt[3,]
matplot(agem,cbind(bk,med,hi),log="y",type='b',ylab="cases per person-year",xlab="age at diagnosis",lty=1,col=rb,main="CML Incidence (Hiroshima males)",cex=1)
matlines(agem,cbind(Ebk,Emed,Ehi),type='l',lty=2,col=rb)
legend("topright",legend=c("low dose data","medium dose data","high dose data","low dose fit","medium dose fit","high dose fit"),col=c(rb,rb),lty=c(1,1,1,2,2,2),bty="n")



#***************************** Block 6 (explore LOW DOSE HYPERSENSITIVITY implications for net lifetime CML risk) *********************************************

ageX=40  # do everything for an exposed 40 year old
agef=seq(ageX,100,1)

# first find survival probability at age a>x given exposed (and alive) at age x 
lt<-read.table("lifeUSA2001.txt",header=T)
m=lt[lt$Sex=="males",];m
mage=c(0.5,3,seq(7.5,97.5,5),105)
m=m[mage>40,];m
mu=m$nqx # mortality rate
mu=rep(mu,each=5)
mu=mu[1:61]/5
mu
Mu=cumsum(mu)
par(mfrow=c(2,2))
S=exp(-Mu);S
plot(agef,S,type="l",xlab="Age",ylab="Survival Probability",main="US Population in 2001")
# which looks about right

#          c1            k           c2           kt 
#-12.78514425   0.04062961  -8.39283620   0.40370903 

X0=c(c1=-12.8,k=.041,c2=-8.39,kt=0.404,boa=.45/3.64, alfkr=.290,betak=0.023,G=40,Dc=0.05)
attach(as.list(X0))

j=1
R=NULL
gammaD=seq(0,4,.001)
for (Dg in gammaD) # assume zero neutrons
{
R[j]=sum(((exp(c1+k*agef) + (Dg+boa*Dg^2)*(agef-ageX)^2*exp(c2-kt*(agef-ageX)) )* exp( -(alfkr*G*exp(-Dg/Dc)+alfkr*(1-exp(-Dg/Dc)))*Dg-betak*Dg^2)-exp(c1+k*agef))*S  )
j=j+1
}

plot(gammaD,R,type="l",xlab="Dose (Gy)",ylab="Net lifetime excess risk of CML",main="CML risk")


plot(gammaD,R,type="l",xlab="Dose (Gy)",ylab="Net lifetime excess risk of CML",main="CML risk",xlim=c(0,.3))

Dg=seq(0,4,.001)
CK=exp(-(alfkr*G*exp(-Dg/Dc)+alfkr*(1-exp(-Dg/Dc)))*Dg-betak*Dg^2)
plot(Dg,CK,type="l",xlab="Dose (Gy)",ylab="Target cell survival probability",main="Cell killing low dose hypersensitivity",log="y")



detach(as.list(X0))

par(mfrow=c(1,1))

